import logging

logging.basicConfig(level=logging.DEBUG,
                    format='[%(asctime)s] %(message)s',
                    datefmt='%Y:%m:%d %H:%M:%S')
logger = logging.getLogger('running')
